local WM = WINDOW_MANAGER;

function AuraMastery:InitializeControls()
	self.aura = {}
	for k, v in pairs(self.svars.auraData) do
		self:CreateAuraControl(k);
		self:RegisterAbilityIds(k);
		self:LoadAura(k);
	end
	self:CreateMenu();
end

function AuraMastery:ReloadAuraControl(auraName)
	self:CreateAuraControl(auraName);
	self:UnloadAura(auraName);
	self:RegisterAbilityIds(auraName);
	self:LoadAura(auraName);
end

function AuraMastery:UnregisterAbilityIds(auraName)

end

function AuraMastery:CreateAuraControl(auraName)
	self.aura[auraName] = WM:GetControlByName(auraName);

	if self.aura[auraName] then
		self.aura[auraName].bg 		= WM:GetControlByName(auraName.."_Backdrop");
		self.aura[auraName].label	= WM:GetControlByName(auraName.."_Label");
		self.aura[auraName].icon	= WM:GetControlByName(auraName.."_Icon");
		self.aura[auraName]:SetHidden(false);
	else
		self.aura[auraName] 			= WM:CreateTopLevelWindow(auraName);
		self.aura[auraName].bg 		= WM:CreateControl("$(parent)_Backdrop", self.aura[auraName], CT_BACKDROP);
		self.aura[auraName].label	= WM:CreateControl("$(parent)_Label", self.aura[auraName], CT_LABEL);
		self.aura[auraName].icon	= WM:CreateControl("$(parent)_Icon", self.aura[auraName], CT_TEXTURE);
	end

	--self.aura[auraName] = WM:CreateTopLevelWindow(auraName);
	self.aura[auraName]:SetHandler("OnMoveStop", function() self:OnDisplayPosChange(auraName) end);
	self.aura[auraName]:SetClampedToScreen(true);
	self.aura[auraName]:SetDimensions(self.svars.auraData[auraName]['width'], self.svars.auraData[auraName]['height']);
	self.aura[auraName]:ClearAnchors();
	self.aura[auraName]:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, self.svars.auraData[auraName]['posX'], self.svars.auraData[auraName]['posY']);
	self.aura[auraName]:SetAlpha(self.svars.auraData[auraName]['alpha']);
	self.aura[auraName]:SetMovable(false);
	self.aura[auraName]:SetHidden(true);
	self.aura[auraName]:SetMouseEnabled(false);

	-- Background and border
	--self.aura[auraName].bg = WM:CreateControl("$(parent)_Backdrop", self.aura[auraName], CT_BACKDROP);
	self.aura[auraName].bg:SetAnchorFill();
	self.aura[auraName].bg:SetDrawLayer(1);
	self.aura[auraName].bg:SetCenterColor(self.svars.auraData[auraName].bgColor[1], self.svars.auraData[auraName].bgColor[2], self.svars.auraData[auraName].bgColor[3], self.svars.auraData[auraName].bgColor[4]);
	self.aura[auraName].bg:SetEdgeTexture(nil, 1, 1, 1, 0);
	self.aura[auraName].bg:SetEdgeColor(self.svars.auraData[auraName].borderColor[1], self.svars.auraData[auraName].borderColor[2], self.svars.auraData[auraName].borderColor[3], self.svars.auraData[auraName].borderColor[4]);	

	-- Cooldown Text
	--self.aura[auraName].label = WM:CreateControl("$(parent)_Label", self.aura[auraName], CT_LABEL);
	self.aura[auraName].label:SetAnchor(TOPLEFT, self.aura[auraName], TOPLEFT, 2,2);
	self.aura[auraName].label:SetAnchor(BOTTOMRIGHT, self.aura[auraName], BOTTOMRIGHT, -2,-2);	
	self.aura[auraName].label:SetVerticalAlignment(1);
	self.aura[auraName].label:SetHorizontalAlignment(1);
	self.aura[auraName].label:SetColor(1,1,1,1);
	self.aura[auraName].label:SetFont("$(BOLD_FONT)|"..tostring(self.svars.auraData[auraName].fontSize).."|outline");

	-- Icon
	--self.aura[auraName].icon = WM:CreateControl("$(parent)_Icon", self.aura[auraName], CT_TEXTURE);
	self.aura[auraName].icon:SetAnchor(CENTER, self.aura[auraName], CENTER, 0,0);
	self.aura[auraName].icon:SetDimensions(self.svars.auraData[auraName]['width'], self.svars.auraData[auraName]['height']);
	self.aura[auraName].icon:SetTexture(self.svars.auraData[auraName].iconPath);
end

function AuraMastery:RemoveAuraControl(auraName)
	self.aura[auraName]:SetHidden(true);	-- Auraanzeige verstecken (NICHT das Element der Menüliste!)
	self.aura[auraName] = nil;
end

function AuraMastery:OnDisplayPosChange(bar)
	self.svars.auraData[bar].posX = self.aura[bar]:GetLeft();
	self.svars.auraData[bar].posY = self.aura[bar]:GetTop();
end

function AuraMastery:LockBar(bar, isLocked)
	self.aura[bar]:SetMouseEnabled(not isLocked)
	self.aura[bar]:SetMovable(not isLocked)
end

function AuraMastery:RegisterAbilityIds(auraName)
	local auraData = self.svars.auraData[auraName];
	local n = #AuraMastery.svars.auraData[auraName].triggers;
	for i = 1, n do

		if ("true" == self.svars.auraData[auraName].triggers[i].checkAll) then
			local lang = GetCVar("language.2");
			local spellId = self.svars.auraData[auraName].triggers[i].spellId;
			local name = zo_strformat("<<z:1>>", GetAbilityName(spellId));
			if (self.abilityData[lang][name]) then
				for abilityRank, abilityId in pairs(self.abilityData[lang][name]) do
					self:AddAbilityId(abilityId, auraName);
				end
			end
		else
			self:AddAbilityId(AuraMastery.svars.auraData[auraName].triggers[i].spellId, auraName);
		end
	end
end

function AuraMastery:AddAbilityId(abilityId, auraName)
	if nil == self.trackedEvents['EVENT_EFFECT_CHANGED']['abilityIds'][abilityId] then
		self.trackedEvents['EVENT_EFFECT_CHANGED']['abilityIds'][abilityId] = {};
	end
	self.trackedEvents['EVENT_EFFECT_CHANGED']['abilityIds'][abilityId] = auraName;
	--[[
	if (nil == self.auraAbilityIds[abilityId]) then
		self.auraAbilityIds[abilityId] = {};
	else
		table.insert(self.auraAbilityIds[abilityId], auraName);
	end
	]]
end

function AuraMastery:InTable(t, e)
	for k,v in pairs(t) do	
		if (v==e) then
			return true;
		end
	end
	return false;
end

function AuraMastery:LoadAura(auraName)
	table.insert(self.loadedAuras, auraName);
end

function AuraMastery:UnloadAura(auraName)
	local tLength = #self.loadedAuras;
	for i=1, tLength do
		if (self.loadedAuras[i] == auraName) then
			table.remove(self.loadedAuras, i);
			--d("Die Aura "..auraName.." mit dem Key "..i.." wurde aus loadedAuras entfernt.");
		end
	end
end

function AuraMastery:UnloadAuras()
	for i = 1, #self.loadedAuras do
		self.loadedAuras[i] = nil;
	end
end